"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _postcss = require("postcss");

var _utils = require("../../utils");

var ruleName = "rule-properties-order";

exports.ruleName = ruleName;
var messages = (0, _utils.ruleMessages)(ruleName, {
  expected: function expected(first, second) {
    return "Expected property \"" + first + "\" to come before property \"" + second + "\"";
  }
});

exports.messages = messages;

exports["default"] = function (expectation) {
  return function (root, result) {

    // Shallow loop
    root.each(function (node) {
      if (node.type === "rule" || node.type === "at-rule") {
        checkInNode(node);
      }
    });

    function checkInNode(node) {

      var previousProp = {};
      var isFirstDecl = true;

      node.each(function (child) {

        if (child.nodes && child.nodes.length) {
          checkInNode(child);
        }

        if (child.type !== "decl") {
          return;
        }

        var prop = {
          name: child.prop,
          unprefixedName: _postcss.vendor.unprefixed(child.prop)
        };

        // Skip first decl
        if (isFirstDecl) {
          isFirstDecl = false;
          previousProp = prop;
          return;
        }

        // Same unprefixed property name
        if (prop.unprefixedName === previousProp.unprefixedName && prop.name >= previousProp.name) {
          previousProp = prop;
          return;
        }

        // Different unprefixed property names
        if (prop.unprefixedName !== previousProp.unprefixedName) {

          // Alphabetical
          if (expectation === "alphabetical" && prop.unprefixedName >= previousProp.unprefixedName) {
            previousProp = prop;
            return;
          }

          // Array of properties
          if (Array.isArray(expectation)) {

            var propIndex = expectation.indexOf(prop.unprefixedName);
            var previousPropIndex = expectation.indexOf(previousProp.unprefixedName);

            // Check that two known properties are in order
            if (propIndex !== -1 && previousPropIndex !== -1 && propIndex >= previousPropIndex) {
              previousProp = prop;
              return;
            }

            // Skip over unknown properties as any subsequent known properties will flag
            if (propIndex === -1) {
              previousProp = prop;
              return;
            }
          }
        }

        (0, _utils.report)({
          message: messages.expected(prop.name, previousProp.name),
          node: child,
          result: result,
          ruleName: ruleName
        });

        previousProp = prop;
      });
    }
  };
};