"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _postcss = require("postcss");

var _postcss2 = _interopRequireDefault(_postcss);

var _tape = require("tape");

var _tape2 = _interopRequireDefault(_tape);

var _disableRanges = require("../disableRanges");

/**
 * Create a ruleTester for a specified rule.
 *
 * The ruleTester is a function accepting options and a callback.
 * The callback is passed on object exposing functions that
 * check CSS strings against the rule with the specified options.
 *
 * `ruleTester.only()` has the same API but uses `test.only()` from
 * tape -- meaning that those tests run exclusively.
 *
 * @param {function} rule
 * @param {string} ruleName
 * @return {function} ruleTester for the specified rule
 */

var _disableRanges2 = _interopRequireDefault(_disableRanges);

exports["default"] = function (rule, ruleName) {
  var ruleTesterInstance = function ruleTesterInstance(primaryOptions, secondaryOptions, cb) {
    if (typeof secondaryOptions === "function") {
      cb = secondaryOptions;
      secondaryOptions = null;
    }
    return buildRuleTester(_tape2["default"], primaryOptions, secondaryOptions, cb);
  };

  ruleTesterInstance.only = function (primaryOptions, secondaryOptions, cb) {
    if (typeof secondaryOptions === "function") {
      cb = secondaryOptions;
      secondaryOptions = null;
    }
    return buildRuleTester(_tape2["default"].only, primaryOptions, secondaryOptions, cb);
  };

  return ruleTesterInstance;

  function buildRuleTester(testFn, primaryOptions, secondaryOptions, cb) {
    var optionsString = JSON.stringify(primaryOptions);
    if (secondaryOptions) {
      optionsString += ", " + JSON.stringify(secondaryOptions);
    }
    testFn(ruleName.toUpperCase() + ": " + optionsString, function (t) {
      cb({

        /**
         * Checks that a CSS string is valid according to the
         * specified rule/options.
         *
         * @param {string} cssString
         */
        ok: function ok(cssString, description) {
          t.test("pass: " + JSON.stringify(cssString), function (st) {
            postcssProcess(cssString, function (result) {
              st.equal(result.warnings().length, 0, description + " should pass");
              st.end();
            });
          });
        },

        /**
         * Checks that a CSS string is INVALID according to the
         * specified rule/options -- that a warning is registered
         * with the expected warning message.
         *
         * @param {string} cssString
         * @param {string} warningMessage
         * @param {string} [description]
         */
        notOk: function notOk(cssString, warningMessage, description) {
          t.test("fail: " + JSON.stringify(cssString), function (st) {
            postcssProcess(cssString, function (result) {
              var warnings = result.warnings();
              st.equal(warnings.length, 1, description + " should warn");
              if (warnings.length === 1) {
                var finishedDescription = description ? description + " should report \"" + warningMessage + "\"" : "should report \"" + warningMessage + "\"";
                st.equal(warnings[0].text, warningMessage, finishedDescription);
              } else {
                st.pass("no warning to test");
              }
              st.end();
            });
          });
        }
      });
      t.end();
    });

    function postcssProcess(cssString, callback) {
      (0, _postcss2["default"])()
      // First add disabledRanges to the result
      .use(_disableRanges2["default"])
      // Then run the rule
      .use(rule(primaryOptions, secondaryOptions)).process(cssString).then(callback)["catch"](function (err) {
        return console.error(err.stack);
      });
    }
  }
};

module.exports = exports["default"];